/*
*    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
*    
*    Permission is hereby granted, free of charge, to any person obtaining
*    a copy of this software and associated documentation files 
*    (curlpp), to deal in the Software without restriction, 
*    including without limitation the rights to use, copy, modify, merge,
*    publish, distribute, sublicense, and/or sell copies of the Software,
*    and to permit persons to whom the Software is furnished to do so, 
*    subject to the following conditions:
*    
*    The above copyright notice and this permission notice shall be included
*    in all copies or substantial portions of the Software.
*    
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
*    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <string.h>
#include <fstream>

#include "curlpp.hpp"
#include "registries.hpp"

int main()
{

   std::string HTTP_URL1("http://linuxfr.org");
   std::string HTTP_URL2("http://www.slashdot.org");

   curlpp::initialize();
   
   try
   {
      
      //we construct our requests.
      curlpp::easy request1;
      curlpp::easy request2;
      
      //we construct our url
      curlpp::registries::url my_url;
      my_url.set( HTTP_URL1 );

      //curlpp::registry_manager< curlpp::easy > my_manager;
      
      //we register our requests to use the my_url.
      my_url.record( request1 );
      my_url.record( request2 );
      
      std::cout << "request with " << HTTP_URL1 << std::endl;
      request1.perform();
      std::cout << "request with " << HTTP_URL1 << std::endl;
      request2.perform();
      
      //we change all registered request to my_url to
      //use HTTP_URL2
      my_url.set( HTTP_URL2 );
      
      std::cout << "request with " << HTTP_URL2 << std::endl;
      request1.perform();
      std::cout << "request with " << HTTP_URL2 << std::endl;
      request2.perform();
      
      //we just change request1 to use HTTP_URL1
      request1.url( HTTP_URL1 );
      
      std::cout << "request with " << HTTP_URL1 << std::endl;
      request1.perform();
      std::cout << "request with " << HTTP_URL2 << std::endl;
      request2.perform();
      
      //we change all registered request to my_url to use
      //HTTP_URL2. Because request1 is still registered in
      //my_url, request1 will be changed to use HTTP_URL2 too.
      my_url.set( HTTP_URL2 );
      
      std::cout << "request with " << HTTP_URL2 << std::endl;
      request1.perform();
      std::cout << "request with " << HTTP_URL2 << std::endl;
      request2.perform();
      
      //we unregister request1 from my_url
      my_url.unrecord( request1 );
      
      //we change all registered request to my_url to use
      //HTTP_URL1. Because request1 is not longer registered in
      //my_url, request1 will still use HTTP_URL2.
      my_url.set( HTTP_URL1 );
      
      std::cout << "request with " << HTTP_URL2 << std::endl;
      request1.perform();
      std::cout << "request with " << HTTP_URL1 << std::endl;
      request2.perform();
      
   }
   catch ( curlpp::exception & e )
   {
      std::cout << e.what() << std::endl;
   }
   
   curlpp::terminate();
  
   std::cout << "example 2 finished" << std::endl;
   
   return EXIT_SUCCESS;
}
