#include <iostream> 
/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <iostream>

#include "curlpp.hpp"
#include "registries.hpp"

#define HTTP_URL "http://www.slashdot.org/"

int main()
{
   try
   {
      curlpp::cleanup cleaner;
      
      curlpp::easy request1;
      request1.url( HTTP_URL );
      
      curlpp::easy request2;
      request2.url( HTTP_URL );
      
      //we construct our progress to set it to true
      curlpp::registries::progress my_progress;
      my_progress.set( true );
      
      //we register our request to our progress
      my_progress.record( request1 );
      my_progress.record( request2 );
      
      std::cout << "performing with the progress to true" << std::endl;
      request1.perform();
      std::cout << "performing with the progress to true" << std::endl;
      request2.perform();
      
      //we change all request registered to my_progress to false
      my_progress.set( false );
      
      std::cout << "performing with the progress to false" << std::endl;
      request1.perform();
      std::cout << "performing with the progress to false" << std::endl;
      request2.perform();
      
      //we just change the request1 to be progress
      request1.progress( true );
      
      std::cout << "performing with the progress to true" << std::endl;
      request1.perform();
      std::cout << "performing with the progress to false" << std::endl;
      request2.perform();
      
      //we change all request registered to my_progress
      //to use INTERFACE2. Because we don't have unregistered
      //request1 to my_progress, request1 will be set to
      //INTERFACE2
      my_progress.set( false );
      
      std::cout << "performing with the progress to false" << std::endl;
      request1.perform();
      std::cout << "performing with the progress to false" << std::endl;
      request2.perform();
      
      //we now unregister request1 to my_progress
      my_progress.unrecord( request1 );
      
      //we change all request registered to my_progress to
      //be progress so request1 will not be changed.
      my_progress.set( true );
      
      std::cout << "performing with the progress to false" << std::endl;
      request1.perform();
      std::cout << "performing with the progress to true" << std::endl;
      request2.perform();
   }
   catch ( curlpp::exception & e )
   {
      std::cout << e.what() << std::endl;
   }
}
