#include <iostream> 
/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <iostream>

#include "curlpp.hpp"
#include "registries.hpp"

#define HTTP_URL "http://www.slashdot.org/"

#define INTERFACE1 "eth0"
#define INTERFACE2 "eth0:1"

int main()
{
   try
   {
      curlpp::cleanup cleaner;
      
      curlpp::easy request1;
      request1.url( HTTP_URL );
      request1.verbose();
      
      curlpp::easy request2;
      request2.url( HTTP_URL );
      request2.verbose();
      
      //we construct our interface to use the INTERFACE1
      curlpp::registries::interface my_interface;
      my_interface.set( INTERFACE1 );
      
      //we register our request to our interface
      my_interface.record( request1 );
      my_interface.record( request2 );
      
      std::cout << "performing with the INTERFACE1 interface" << std::endl;
      request1.perform();
      std::cout << "performing with the INTERFACE1 interface" << std::endl;
      request2.perform();
      
      //we change all request registered to my_interface to use
      //INTERFACE2
      my_interface.set( INTERFACE2 );
      
      std::cout << "performing with the INTERFACE2 interface" << std::endl;
      request1.perform();
      std::cout << "performing with the INTERFACE2 interface" << std::endl;
      request2.perform();
      
      //we just change the request1 to use the INTERFACE1
      request1.interface( INTERFACE1 );
      
      std::cout << "performing with the INTERFACE1 interface" << std::endl;
      request1.perform();
      std::cout << "performing with the INTERFACE2 interface" << std::endl;
      request2.perform();
      
      //we change all request registered to my_interface
      //to use INTERFACE2. Because we don't have unregistered
      //request1 to my_interface, request1 will be set to
      //INTERFACE2
      my_interface.set( INTERFACE2 );
      
      std::cout << "performing with the INTERFACE2 interface" << std::endl;
      request1.perform();
      std::cout << "performing with the INTERFACE2 interface" << std::endl;
      request2.perform();
      
      //we now unregister request1 to my_interface
      my_interface.unrecord( request1 );
      
      //we change all request registered to my_interface to
      //use INTERFACE1, so request1 will not be changed.
      my_interface.set( INTERFACE1 );
      
      std::cout << "performing with the INTERFACE2 interface" << std::endl;
      request1.perform();
      std::cout << "performing with the INTERFACE1 interface" << std::endl;
      request2.perform();
   }
   catch ( curlpp::exception & e )
   {
      std::cout << e.what() << std::endl;
   }
}
