#include <iostream> 
/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <iostream>

#include "curlpp.hpp"
#include "registries.hpp"
#include "registerer.hpp"

#define HTTP_URL "http://www.slashdot.org/"

int main()
{
   try
   {
      curlpp::cleanup cleaner;
      
      curlpp::registerer< curlpp::easy > my_registerer;
      curlpp::registries::url my_url;
      curlpp::registries::verbose my_verbose;
      my_registerer.record_registry( &my_url );
      my_registerer.record_registry( &my_verbose );
      
     
      //we contruct our request and regiter them to my_registerer, in fact
      //they will register to my_url and my_verbose.
      curlpp::easy request1;
      my_registerer.record( request1 );
      curlpp::easy request2;
      my_registerer.record( request2 );
      
      my_url.set( HTTP_URL );
      my_verbose.set( true );
      
      //we construct our header to use the true
      curlpp::registries::header my_header;
      my_header.set( true );
      
      //we register our request to our header
      my_header.record( request1 );
      my_header.record( request2 );
      
      std::cout << "performing with the header to true" << std::endl;
      request1.perform();
      std::cout << "performing with the header to true" << std::endl;
      request2.perform();
      
      //we change all request registered to my_header to use
      //false
      my_header.set( false );
      
      std::cout << "performing with the header to false" << std::endl;
      request1.perform();
      std::cout << "performing with the header to false" << std::endl;
      request2.perform();
      
      //we just change the request1 to use the true
      request1.header( true );
      
      std::cout << "performing with the header to true" << std::endl;
      request1.perform();
      std::cout << "performing with the header to false" << std::endl;
      request2.perform();
      
      //we change all request registered to my_header
      //to use false. Because we don't have unregistered
      //request1 to my_header, request1 will be set to
      //false
      my_header.set( false );
      
      std::cout << "performing with the header to false" << std::endl;
      request1.perform();
      std::cout << "performing with the header to false" << std::endl;
      request2.perform();
      
      //we now unregister request1 to my_header
      my_header.unrecord( request1 );
      
      //we change all request registered to my_header to
      //use true, so request1 will not be changed.
      my_header.set( true );
      
      std::cout << "performing with the header to false" << std::endl;
      request1.perform();
      std::cout << "performing with the header to true" << std::endl;
      request2.perform();
   }
   catch ( curlpp::exception & e )
   {
      std::cout << e.what() << std::endl;
   }
}
